######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

#  BRUGS program for hierarchical modeling of successive
#  removal data combined with abundance indices to improve
#  estimations of A. salmon juveniles abundance.
#  Data from the Nivelle River, France
#  Chapter 10 - Section 10.2

######################################################################################
######################################################################################



rm(list = ls())

library(boot)
library("BRugs")
library(coda)
library(MASS)

# MCMC config.
# -------------------------------------------

NburnIn <- 10000
Niter <- 10000
thin <- 10



# ---------------------------------------------------------------
#  INDEPENDENT model
# ---------------------------------------------------------------

model <- "model_Hierarchical_SuccRem2_Nivelle_ind.txt"
data <- "data_Hierarchical_SuccRem2_Nivelle.txt"
inits <- "inits_Hierarchical_SuccRem2_Nivelle_ind.txt"

modelCheck(model)  
modelData(data)   
modelCompile(numChains=1)      
modelInits(inits)

# Burn-in period

modelUpdate(NburnIn) 

# Inferences

monitor <- c("pi","d","C3")
samplesSet(monitor) 
modelUpdate(numUpdates = Niter, thin = thin)

# Store MCMC samples of theta in a data.frame

pi.ind <- as.data.frame(as.matrix(buildMCMC("pi")))
d.ind <- as.data.frame(as.matrix(buildMCMC("d")))  	
C3.ind <- as.data.frame(as.matrix(buildMCMC("C3"))) 	



# ---------------------------------------------------------------
#  HBM model
# ---------------------------------------------------------------

model <- "model_Hierarchical_SuccRem2_Nivelle_hbm.txt"
data <- "data_Hierarchical_SuccRem2_Nivelle.txt"
inits <- "inits_Hierarchical_SuccRem2_Nivelle_hbm.txt"

modelCheck(model)  
modelData(data)   
modelCompile(numChains=1)      
modelInits(inits)

# Burn-in period

modelUpdate(NburnIn, thin = 1) 

# Inferences

monitor <- c("pi","d","a","b","c1","d1","mu.d","mu.p","sd.d","sd.p","C3")
samplesSet(monitor)
modelUpdate(numUpdates = Niter, thin = thin)


# Store MCMC samples of theta in a data.frame

pi.hbm <- as.data.frame(as.matrix(buildMCMC("pi")))
d.hbm <- as.data.frame(as.matrix(buildMCMC("d")))  	
C3.hbm <- as.data.frame(as.matrix(buildMCMC("C3"))) 	
a.hbm <- samplesSample("a")
b.hbm <- samplesSample("b")
c1.hbm <- samplesSample("c1")
d1.hbm <- samplesSample("d1")
mu.d.hbm <- samplesSample("mu.d")
mu.p.hbm <- samplesSample("mu.p")
sd.d.hbm <- samplesSample("sd.d")
sd.p.hbm <- samplesSample("sd.p")





# ---------------------------------------------------------------
# Model with successive removals combined with Abundance Indices
# ---------------------------------------------------------------

model <- "model_Hierarchical_SuccRem2_Nivelle_AI.txt"
data <- "data_Hierarchical_SuccRem2_Nivelle.txt"
data.AI <- "data_Hierarchical_SuccRem2_Nivelle_AI.txt"
inits <- "inits_Hierarchical_SuccRem2_Nivelle_AI.txt"

modelCheck(model)  
modelData(data)
modelData(data.AI)
modelCompile(numChains=1)      
modelInits(inits)
modelGenInits()

# Burn-in period

modelUpdate(NburnIn) 

# Inferences

monitor <- c("pi","d","mu.p","sd.p","mu.d","sd.d","phi","k","dpred")
samplesSet(monitor) 
modelUpdate(numUpdates = Niter, thin = thin) 

d.AI <- as.data.frame(as.matrix(buildMCMC("d")))
dpred <- as.data.frame(as.matrix(buildMCMC("dpred")))
mu.d.AI <- samplesSample("mu.d")
mu.p.AI <- samplesSample("mu.p")
sd.d.AI <- samplesSample("sd.d")
sd.p.AI <- samplesSample("sd.p")
phi <- samplesSample("phi")
k <- samplesSample("k")





# ---------------------------------------------------------------
#  Figures
# ---------------------------------------------------------------


# HBM - Marginal Posterior with pair-wise correlations
# ----------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Fig1chap7SuccRemovHierPost.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

x1 <- a.hbm
x2 <- b.hbm
x3 <- c1.hbm
x4 <- d1.hbm
x5 <- mu.p.hbm
x6 <- mu.d.hbm
x <- cbind(x1,x2,x3,x4,x5,x6)
labels <- c("a","b","c","d",expression(mu[pi]),expression(mu[delta]))

# put histograms on the diagonal
panel.hist <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- hist(x,50, plot = T,freq=F)
    breaks <- h$breaks; nB <- length(breaks)
    y <- h$counts; y <- y/max(y)
    rect(breaks[-nB], 0, breaks[-1], y, col="blue", ...)
}
panel.hist <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- density(x,adjust=1)
    h$y<-h$y/max(h$y)
    xlim=range(h$x)
    lines(h, col="blue", ...)
}

# put (absolute) correlations on the upper panels
panel.cor <- function(x, y, digits=2, prefix="", cex.cor)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- (cor(x, y))
    txt <- format(c(r, 0.123456789), digits=digits)[1]
    txt <- paste(prefix, txt, sep="")
    if(missing(cex.cor)) cex <- 0.8/strwidth(txt)
    text(0.5, 0.5, txt, cex = 1.5)
}
par(pch='.')
pairs(x=x,labels=labels, lower.panel=panel.smooth,diag.panel=panel.hist, upper.panel=panel.cor, cex.labels = 2, font.labels=1.5)

dev.off()

par(def.par)




# Abundance indices vs estimated densities
# ----------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Fig1chap7SuccRemovHiervsIA.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(bty="n", mar=c(6,5,1,1))

x.label <- "Abundance indices"
y.label <- "Estimated fish density"

x <- source(data.AI)$value$IA
y <- apply(as.matrix(d.hbm),2,median)

plot(x,y, xlab="", ylab="", xlim = c(0,70), ylim=c(0,0.4), xaxt="n", yaxt = "n", pch=21, bg="black")

axis(side =1, las=1, cex.axis=1)
axis(side =2, las=1, cex.axis=1)
mtext(x.label, line = 3, side = 1, cex=1.3)
mtext(y.label, line = 3.5, side = 2, cex=1.3)

dev.off()

par(def.par)



# Compare the posterior distributions of the densities in the 3 models
# --------------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Fig1chap7SuccRemovfishdens3situations.png"
png(filename = name_figure, height = 500*res, width = 800*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

n <- 33

size.labels = 1
size.text = 1.3

col1 <- "white"
col2 <- "grey75"
col3 <- "grey45"

box.size = 0.4

names <- paste(1:n)
labels <- names
x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 0.5
y.label = "Fish density"
x.label = "Site index"
title = ""

X1 <- d.ind
X2 <- d.hbm
X3 <- d.AI

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1-0.2, outpch = NA, boxwex=box.size, col = col1)
for (i in 2:n) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = i-0.2, add = T, outpch = NA, boxwex=box.size, col = col1) }

for (i in 1:n) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size, col = col2) }

for (i in 1:n) {
boxplot(X3[,i], xaxt = "n", yaxt = "n", at = i+0.2, add = T, outpch = NA, boxwex=box.size, col = col3) }

axis(side =1, at=1:n, labels = labels, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)

dev.off()

par(def.par)



# Joint posterior distribution for (pi,d)  -  INDEPENDENT model
# -------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Fig1chap7SuccRemov_pd_ind.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

x <- d.ind[,5]
y <- pi.ind[,5]

indice <- which(x<=quantile(x, prob = 0.995))
x <- x[indice]
y <- y[indice]

xrange <- range(x) ; nbreaks.x=100
yrange <- range(y) ; nbreaks.y=100

nb.points = 2000

xhist <- hist(x, breaks=seq(xrange[1],xrange[2],length.out=nbreaks.x), plot=FALSE)
yhist <- hist(y, breaks=seq(yrange[1],yrange[2],length.out=nbreaks.y), plot=FALSE)

nf <- layout(matrix(c(2,0,1,3),2,2,byrow=TRUE), c(3,1), c(1,3), TRUE)
layout.show(nf)

par(mar=c(5,5,1,1))
plot(x[1:nb.points], y[1:nb.points], xlim=xrange, ylim=yrange, 
	xlab="", ylab="", pch=".", cex.lab = 1.5)
dens2d <- kde2d(x=x, y=y, n = 100)
contour(dens2d , nlevels = 3, drawlabels=F, col = "blue", lw = 2, add = T) 
mtext(text=expression(delta), side=1, line=3, cex=1.3)
mtext(text=expression(pi), side=2, line=3, cex=1.3)

par(mar=c(0,5,3,1))
barplot(xhist$density, axes=FALSE, space=0, horiz=FALSE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for ",delta)), side=3, line=1, cex=1.3)

par(mar=c(5,0,1,3))
barplot(yhist$density, axes=FALSE, space=0, horiz=TRUE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for ",pi)), side=4, line = 1, las = 0, cex=1.3)

par(def.par)

dev.off()





# Joint posterior distribution for (pi,d)  -  HIERARCHICAL model
# --------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Fig1chap7SuccRemov_pd_hbm.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

x <- d.hbm[,5]
y <- pi.hbm[,5]

indice <- which(x<=quantile(x, prob = 0.995))
x <- x[indice]
y <- y[indice]

xrange <- range(x) ; nbreaks.x=100
yrange <- range(y) ; nbreaks.y=100

nb.points = 2000

xhist <- hist(x, breaks=seq(xrange[1],xrange[2],length.out=nbreaks.x), plot=FALSE)
yhist <- hist(y, breaks=seq(yrange[1],yrange[2],length.out=nbreaks.y), plot=FALSE)

nf <- layout(matrix(c(2,0,1,3),2,2,byrow=TRUE), c(3,1), c(1,3), TRUE)
layout.show(nf)

par(mar=c(5,5,1,1))
plot(x[1:nb.points], y[1:nb.points], xlim=xrange, ylim=yrange, 
	xlab="", ylab="", pch=".", cex.lab = 1.5)
dens2d <- kde2d(x=x, y=y, n = 100)
contour(dens2d , nlevels = 3, drawlabels=F, col = "blue", lw = 2, add = T) 
mtext(text=expression(delta), side=1, line=3, cex=1.3)
mtext(text=expression(pi), side=2, line=3, cex=1.3)

par(mar=c(0,5,3,1))
barplot(xhist$density, axes=FALSE, space=0, horiz=FALSE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for ",delta)), side=3, line=1, cex=1.3)

par(mar=c(5,0,1,3))
barplot(yhist$density, axes=FALSE, space=0, horiz=TRUE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for ",pi)), side=4, line = 1, cex=1.3, las=0)

par(def.par)

dev.off()





# HBM+AI - Marginal Posterior with pair-wise correlations
# -------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Fig1chap7SuccRemovIApredChgparam.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

x1 <- mu.d.AI
x2 <- sd.d.AI
x3 <- mu.p.AI
x4 <- sd.p.AI
x5 <- phi
x6 <- k
x <- cbind(x1,x2,x3,x4,x5,x6)
labels <- c(expression(mu[delta]),expression(sigma[delta]),expression(mu[pi]),expression(sigma[pi]),expression(phi),"k")

# put histograms on the diagonal
panel.hist <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- hist(x,50, plot = T,freq=F)
    breaks <- h$breaks; nB <- length(breaks)
    y <- h$counts; y <- y/max(y)
    rect(breaks[-nB], 0, breaks[-1], y, col="blue", ...)
}
panel.hist <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- density(x,adjust=1)
    h$y<-h$y/max(h$y)
    xlim=range(h$x)
    lines(h, col="blue", ...)
}

# put (absolute) correlations on the upper panels
panel.cor <- function(x, y, digits=2, prefix="", cex.cor)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- (cor(x, y))
    txt <- format(c(r, 0.123456789), digits=digits)[1]
    txt <- paste(prefix, txt, sep="")
    if(missing(cex.cor)) cex <- 0.8/strwidth(txt)
    text(0.5, 0.5, txt, cex = 1.5)
}

par(pch='.')
pairs(x=x,labels=labels, lower.panel=panel.smooth,diag.panel=panel.hist, upper.panel=panel.cor, cex.labels = 2, font.labels=1.5)

par(def.par)

dev.off()





# Posterior predictive of the densities given observed abundance indices
# ----------------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Fig1chap7SuccRemovDensvsIApred.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(bty="n", mar=c(6,5,1,1))

x.label <- "Abundance indices"
y.label <- "Posterior predictive of fish density"

x <- source(data.AI)$value$IA
y <- apply(as.matrix(d.hbm),2,median)

xx <- source(data.AI)$value$IApred
xx <- c(xx,rev(xx))
qq <- apply(as.matrix(dpred),2,quantile,prob=c(0.1,0.25,0.5,0.75,0.9))

plot(x,y, xlab="", ylab="", xlim = c(0,70), ylim=c(0,0.4), xaxt="n", yaxt = "n", pch=21, bg="black")

yy2 <- c(qq[1,],rev(qq[5,]))
polygon(xx,yy2,col="gray", border = NA)

yy1 <- c(qq[2,],rev(qq[4,]))
polygon(xx,yy1,col="lightgray", border = NA)
lines(1:65,qq[3,], lwd=2)

points(x,y, xlab="", ylab="", xaxt="n", yaxt = "n", pch=21, bg="black")

axis(side =1, las=1, cex.axis=1)
axis(side =2, las=1, cex.axis=1)
mtext(x.label, line = 3, side = 1, cex=1.3)
mtext(y.label, line = 3.5, side = 2, cex=1.3)

par(def.par)

dev.off()



